import sys
from typing import Optional

from .OlvidClient import OlvidClient
from .GrpcTlsConfiguration import GrpcTlsConfiguration


class OlvidBot(OlvidClient):
	"""

	OlvidBot implements a set of method named on_something. There is one method for each gRPC notification method.
	On instantiation overwritten methods will automatically be subscribed as notification listener.
	For example:

	class Bot(OlvidBot)
		async def on_message_received(self, message: datatypes.Message):
			print(message)

	Every time Bot class is instantiated it will add a listener to message_received notification with the method as handler.

	OlvidBot can also add Command objects with add_command method. Command are specific listeners.
	They subclass listeners.MessageReceivedListener, and they are created with a regexp filter that will filter notifications.
	Only messages that match the regexp will raise a notification.
	Commands can be added using OlvidBot.command decorator:
	For example:

	class Bot(OlvidBot)
		@OlvidBot.command(regexp_filter="^!help")
		async def on_message_received(self, message: datatypes.Message):
			await message.reply("Help message")
	"""
	def __init__(self, bot_name: str = None, client_key: Optional[str] = None, server_target: Optional[str] = None,
				parent_client: Optional['OlvidClient'] = None, tls_configuration: GrpcTlsConfiguration = None):
		OlvidClient.__init__(self, client_key=client_key, server_target=server_target, parent_client=parent_client, tls_configuration=tls_configuration)
		self._name = bot_name if bot_name is not None else self.__class__.__name__

		print("WARNING: OlvidBot is marked as deprecated since v1.1.0 and will be removed in future release\nUse OlvidClient instead.", file=sys.stderr)

	# read only properties
	@property
	def name(self) -> str:
		return self._name

	#####
	# Common methods
	#####
	def __str__(self) -> str:
		current_listeners = [f"{listener.listener_key}: (finished: {listener.is_finished})" for listener in self._listeners_set]
		return f"{self.name}: {', '.join(current_listeners)}"

	#####
	# tools
	#####
	def print(self, *args, **kwargs):
		print(f"{self._name}:", *args, **kwargs)



import asyncio

from ..core.OlvidClient import OlvidClient
from .. import datatypes
from .logger import tools_logger


class KeycloakAutoInvitationBot(OlvidClient):
	LAST_LIST_TIMESTAMP_STORAGE_KEY = "keycloak-auto-invitation-bot-last-list-timestamp"

	def __init__(self, parent_client: OlvidClient = None, cron_interval_min: int = 5):
		super().__init__(parent_client=parent_client)

		self.add_background_task(self._cron_task(cron_interval_min))

	async def add_every_new_keycloak_member(self):
		last_list_timestamp_str: str = await self.storage_get(self.LAST_LIST_TIMESTAMP_STORAGE_KEY)
		last_list_timestamp: Optional[int] = int(last_list_timestamp_str) if last_list_timestamp_str else None

		# request all / new keycloak users that are not already contacts
		new_last_list_timestamp = last_list_timestamp
		tools_logger.info(f"{self.__class__.__name__}: last list timestamp: {last_list_timestamp}")
		async for users, new_last_list_timestamp in self.keycloak_user_list(last_list_timestamp=last_list_timestamp if last_list_timestamp else None, filter=datatypes.KeycloakUserFilter(contact=datatypes.KeycloakUserFilter.Contact.CONTACT_IS_NOT)):
			for user in users:
				user: datatypes.KeycloakUser
				tools_logger.info(f"{self.__class__.__name__}: new keycloak user: {user.display_name}")
				await self.keycloak_add_user_as_contact(keycloak_id=user.keycloak_id)

		# update last_list_timestamp in storage
		await self.storage_set(self.LAST_LIST_TIMESTAMP_STORAGE_KEY, str(new_last_list_timestamp))

	async def _cron_task(self, interval_min: int):
		while True:
			tools_logger.info(f"{self.__class__.__name__}: _cron_task: start cron task")
			try:
				await self.add_every_new_keycloak_member()
			except Exception:
				tools_logger.exception(f"{self.__class__.__name__}: _cron_task")
			await asyncio.sleep(interval_min * 60)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: olvid/daemon/command/v1/tool_commands.proto
# Protobuf Python Version: 5.28.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    2,
    '',
    'olvid/daemon/command/v1/tool_commands.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+olvid/daemon/command/v1/tool_commands.proto\x12\x17olvid.daemon.command.v1\"\r\n\x0bPingRequest\"\x0e\n\x0cPingResponseB\xc0\x01\n\x1b\x63om.olvid.daemon.command.v1B\x11ToolCommandsProtoP\x01Z\x0folvid.io/daemon\xa2\x02\x03ODC\xaa\x02\x17Olvid.Daemon.Command.V1\xca\x02\x17Olvid\\Daemon\\Command\\V1\xe2\x02#Olvid\\Daemon\\Command\\V1\\GPBMetadata\xea\x02\x1aOlvid::Daemon::Command::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'olvid.daemon.command.v1.tool_commands_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033com.olvid.daemon.command.v1B\021ToolCommandsProtoP\001Z\017olvid.io/daemon\242\002\003ODC\252\002\027Olvid.Daemon.Command.V1\312\002\027Olvid\\Daemon\\Command\\V1\342\002#Olvid\\Daemon\\Command\\V1\\GPBMetadata\352\002\032Olvid::Daemon::Command::V1'
  _globals['_PINGREQUEST']._serialized_start=72
  _globals['_PINGREQUEST']._serialized_end=85
  _globals['_PINGRESPONSE']._serialized_start=87
  _globals['_PINGRESPONSE']._serialized_end=101
# @@protoc_insertion_point(module_scope)



import logging

from ClientHolder import ClientHolder, ClientWrapper

async def test_tool(client_holder: ClientHolder, fast_mode=False):
	logging.info(f"tools: ping")
	for c in client_holder.clients:
		await c.ping()
